<?php
// @codingStandardsIgnoreStart
final class ControllerModuleRoistat extends Controller {

	public function index() {
        $this->load->model('setting/setting');
        $setting = $this->model_setting_setting->getSetting('roistat');

        $action = array_key_exists('action', $_GET) ? $_GET['action'] : null;
        $token = array_key_exists('token', $_GET) ? $_GET['token'] : null;

        if (!array_key_exists('username', $setting) || !array_key_exists('password', $setting)) {
            exit("Data of authorization aren't set");
        }
        if ($token !== md5($setting['username'].$setting['password'])) {
            exit("Incorrect token");
        }

        if ($action === 'export') {
            $response = $this->exportOfOrders();
        } elseif ($action === 'export_clients') {
            $response = $this->exportOfClients();
        } elseif ($action === 'lead') {
            $response = $this->createLead();
        } else {
            $response = array('status' => 'error', 'message' => 'Action is undefined');
        }

        $output = json_encode($response);
        $this->response->addHeader('Content-Type: application/json; charset=Windows-1251');
        $this->response->setOutput($output);
	}

    /**
     * @return array
     */
	private function exportOfOrders() {
        $result = array('orders' => array(), 'statuses' => array());

        $limit               = isset($_GET['limit']) ? $_GET['limit'] : null;
        $offset              = isset($_GET['offset']) ? $_GET['offset'] : null;
        $edit_date_timestamp = isset($_GET['date']) ? $_GET['date'] : time() - 31 * 24 * 60 * 60;
        if (!is_numeric($edit_date_timestamp) || (int)$edit_date_timestamp < 0) {
            exit('Incorrect request');
        }
        if (($limit !== null && !is_numeric($limit)) || ($offset !== null && !is_numeric($offset))) {
            exit('Incorrect request');
        }
        $edit_datetime       = new \DateTime("@{$edit_date_timestamp}");
        $edit_format_date    = $edit_datetime->format('Y-m-d H:i:s');

        $language_id = $this->config->get('config_language_id');
        $store_id    = $this->config->get('config_store_id');
        $dbPrefix = DB_PREFIX;
        $statuses   = $this->db->query("SELECT * FROM `{$dbPrefix}order_status` WHERE `language_id` = '{$language_id}'");
        $selectQueryBase = 'SELECT * ';
        $countQueryBase = 'SELECT COUNT(*) as total ';
        $orderQueryEnd = "FROM `{$dbPrefix}order` WHERE `store_id` = '{$store_id}' AND `date_modified` > '{$edit_format_date}'";
        $orders_sql = $selectQueryBase . $orderQueryEnd;
        $orders_count_sql = $countQueryBase . $orderQueryEnd;
        if ($limit !== null && $offset !== null) {
            $orders_sql .= " LIMIT {$limit} OFFSET {$offset}";
        }
        $orders = $this->db->query($orders_sql);
        $ordersCount = $this->db->query($orders_count_sql);

        if (array_key_exists('total', $ordersCount->row)) {
            $result['pagination'] = ['total_count' => $ordersCount->row['total']];
        }

        foreach ($statuses->rows as $status) {
            $result['statuses'][] = array(
                'id'   => $status['order_status_id'],
                'name' => $status['name'],
            );
        }
        foreach ($orders->rows as $order) {
            $newOrder = array(
                'id'          => $order['order_id'],
                'date_create' => $order['date_added'],
                'date_update' => $order['date_modified'],
                'status'      => $order['order_status_id'],
                'price'       => $order['total'],
                'visit'       => $order['roistat_visit'],
                'fields'      => array(),
            );
            if (array_key_exists('shipping_method', $order)) {
                $newOrder['fields']['Способ доставки'] = $order['shipping_method'];
            }
            if (array_key_exists('payment_method', $order)) {
                $newOrder['fields']['Способ оплаты'] = $order['payment_method'];
            }
            if (array_key_exists('affiliate_id', $order)) {
                $newOrder['fields']['ID партнера'] = $order['affiliate_id'];
            }
            $result['orders'][] = $newOrder;
        }
        return $result;
	}

    /**
     * @return array
     */
    private function createLead() {
        $customer_id = 0;
        $store_name = $this->config->get('config_name');
        $store_id = $this->config->get('config_store_id');
        $store_url = $this->config->get('config_url');
        $language_id = $this->config->get('config_language_id');
        $order_status_id = $this->config->get('config_order_status_id');

        $title   = $this->db->escape($this->getValue('title'));
        $name    = $this->db->escape($this->getValue('name'));
        $email   = $this->db->escape($this->getValue('email'));
        $phone   = $this->db->escape($this->getValue('phone'));
        $visit   = $this->db->escape($this->getValue('visit'));
        $comment = $this->db->escape($this->getValue('text'));
        $firstname = ($name === '' || $name === null) ? $title : $name;

        $found_customer_id = $this->getCustomerIdByEmail($email);
        if ($found_customer_id) {
            $customer_id = $found_customer_id;
        }

        $sql = "INSERT INTO `" . DB_PREFIX . "order`
                        SET order_status_id = '{$order_status_id}',
                            language_id = '{$language_id}',
                            telephone = '{$phone}',
                            email = '{$email}',
                            firstname = '{$firstname}',
                            customer_id = '{$customer_id}',
                            store_url = '{$store_url}',
                            store_name = '{$title} ({$store_name})',
                            store_id = '{$store_id}',
                            comment = '{$comment}',
                            roistat_visit = '{$visit}',
                            date_added = NOW(),
                            date_modified = NOW()";
        $this->db->query($sql);
        return array('status' => 'ok', 'order_id' => $this->db->getLastId());
    }

    /**
     * @return array
     */
    private function exportOfClients() {
        return array();
    }

    /**
     * @param string $email
     * @return array
     */
    private function getCustomerIdByEmail($email) {
        $query = $this->db->query("SELECT DISTINCT * FROM " . DB_PREFIX . "customer WHERE LCASE(email) = '{$email}'");

        if (count($query->row) > 0) {
            return $query->row['customer_id'];
        } else {
            return false;
        }
    }

    /**
     * @param string $key
     * @return string
     */
    private function getValue($key) {
        return array_key_exists($key, $_GET) ? $_GET[$key] : null;
    }
}
// @codingStandardsIgnoreEnd